@@ -7,10 +7,14 @@ static GHashTable *class_hash;
 static GQuark pygobject_wrapper_key = 0;
 static GQuark pygobject_ownedref_key = 0;
 
+static GList *pygobject_exception_notifiers = NULL;
+
 staticforward PyTypeObject PyGObject_Type;
 static void pygobject_dealloc(PyGObject *self);
 static int  pygobject_traverse(PyGObject *self, visitproc visit, void *arg);
 
+static int  pyg_fatal_exceptions_notify(void);
+
 static void
 object_free(PyObject *op)
 {
@@ -64,7 +68,7 @@ pyg_type_wrapper_dealloc(PyGTypeWrapper *self)
 PyTypeObject PyGTypeWrapper_Type = {
     PyObject_HEAD_INIT(NULL)
     0,
-    "GType",
+    "gobject.GType",
     sizeof(PyGTypeWrapper),
     0,
     (destructor)pyg_type_wrapper_dealloc,
@@ -125,7 +129,7 @@ pyg_param_spec_repr(PyGParamSpec *self)
 
     g_snprintf(buf, sizeof(buf), "<%s '%s'>",
 	       G_PARAM_SPEC_TYPE_NAME(self->pspec),
-	       g_param_get_name(self->pspec));
+	       g_param_spec_get_name(self->pspec));
     return PyString_FromString(buf);
 }
 
@@ -146,21 +150,21 @@ pyg_param_spec_getattr(PyGParamSpec *self, const gchar *attr)
     } else if (!strcmp(attr, "__gtype__")) {
 	return pyg_type_wrapper_new(G_PARAM_SPEC_TYPE(self->pspec));
     } else if (!strcmp(attr, "name")) {
-	const gchar *name = g_param_get_name(self->pspec);
+	const gchar *name = g_param_spec_get_name(self->pspec);
 
 	if (name)
 	    return PyString_FromString(name);
 	Py_INCREF(Py_None);
 	return Py_None;
     } else if (!strcmp(attr, "nick")) {
-	const gchar *nick = g_param_get_nick(self->pspec);
+	const gchar *nick = g_param_spec_get_nick(self->pspec);
 
 	if (nick)
 	    return PyString_FromString(nick);
 	Py_INCREF(Py_None);
 	return Py_None;
     } else if (!strcmp(attr, "blurb") || !strcmp(attr, "__doc__")) {
-	const gchar *blurb = g_param_get_blurb(self->pspec);
+	const gchar *blurb = g_param_spec_get_blurb(self->pspec);
 
 	if (blurb)
 	    return PyString_FromString(blurb);
@@ -180,7 +184,7 @@ pyg_param_spec_getattr(PyGParamSpec *self, const gchar *attr)
 PyTypeObject PyGParamSpec_Type = {
     PyObject_HEAD_INIT(NULL)
     0,
-    "GParamSpec",
+    "gobject.GParamSpec",
     sizeof(PyGParamSpec),
     0,
     (destructor)pyg_param_spec_dealloc,
@@ -232,12 +236,15 @@ pygobject_register_class(PyObject *dict, const gchar *type_name,
 			 PyObject *bases)
 {
     PyObject *o;
-    const char *class_name;
+    const char *class_name, *s;
 
     if (!class_hash)
 	class_hash = g_hash_table_new(g_str_hash, g_str_equal);
 
     class_name = type->tp_name;
+    s = strrchr(class_name, '.');
+    if (s != NULL)
+	class_name = s + 1;
 
     type->ob_type = &PyType_Type;
     if (bases) {
@@ -259,7 +266,6 @@ pygobject_register_class(PyObject *dict, const gchar *type_name,
     if (gtype) {
 	o = pyg_type_wrapper_new(gtype);
 	PyDict_SetItemString(type->tp_dict, "__gtype__", o);
-	PyDict_SetItemString(type->tp_defined, "__gtype__", o);
 	Py_DECREF(o);
     }
 
@@ -404,8 +410,8 @@ pyg_boxed_init(PyGBoxed *self, PyObject *args, PyObject *kwargs)
 static PyTypeObject PyGBoxed_Type = {
     PyObject_HEAD_INIT(NULL)
     0,                                  /* ob_size */
-    "GBoxed",                          /* tp_name */
-    sizeof(PyGBoxed),                  /* tp_basicsize */
+    "gobject.GBoxed",                   /* tp_name */
+    sizeof(PyGBoxed),                   /* tp_basicsize */
     0,                                  /* tp_itemsize */
     /* methods */
     (destructor)pyg_boxed_dealloc,      /* tp_dealloc */
@@ -443,6 +449,7 @@ static PyTypeObject PyGBoxed_Type = {
     PyType_GenericAlloc,		/* tp_alloc */
     PyType_GenericNew,			/* tp_new */
     object_free,			/* tp_free */
+    (inquiry)0,				/* tp_is_gc */
     (PyObject *)0,			/* tp_bases */
 };
 
@@ -473,8 +480,8 @@ pyg_register_boxed(PyObject *dict, const gchar *class_name,
 
     PyDict_SetItemString(type->tp_dict, "__gtype__",
 			 o=pyg_type_wrapper_new(boxed_type));
-    PyDict_SetItemString(type->tp_defined, "__gtype__", o);
     Py_DECREF(o);
+
     g_hash_table_insert(boxed_types, GUINT_TO_POINTER(boxed_type), type);
 
     PyDict_SetItemString(dict, (char *)class_name, (PyObject *)type);
@@ -487,8 +494,8 @@ pyg_boxed_new(GType boxed_type, gpointer boxed, gboolean copy_boxed,
     PyGBoxed *self;
     PyTypeObject *tp;
 
-    g_return_if_fail(boxed_type != 0);
-    g_return_if_fail(!copy_boxed || (copy_boxed && own_ref));
+    g_return_val_if_fail(boxed_type != 0, NULL);
+    g_return_val_if_fail(!copy_boxed || (copy_boxed && own_ref), NULL);
 
     if (!boxed) {
 	Py_INCREF(Py_None);
@@ -517,7 +524,7 @@ pyg_boxed_new(GType boxed_type, gpointer boxed, gboolean copy_boxed,
 static gint
 pyg_enum_get_value(GType enum_type, PyObject *obj, gint *val)
 {
-    GEnumClass *eclass = G_ENUM_CLASS(g_type_class_ref(enum_type));
+    GEnumClass *eclass = NULL;
     gint res = -1;
 
     g_return_val_if_fail(val != NULL, -1);
@@ -528,8 +535,17 @@ pyg_enum_get_value(GType enum_type, PyObject *obj, gint *val)
 	*val = PyInt_AsLong(obj);
 	res = 0;
     } else if (PyString_Check(obj)) {
+	GEnumValue *info;
 	char *str = PyString_AsString(obj);
-	GEnumValue *info = g_enum_get_value_by_name(eclass, str);
+	
+	if (enum_type != G_TYPE_NONE)
+	    eclass = G_ENUM_CLASS(g_type_class_ref(enum_type));
+	else {
+	    PyErr_SetString(PyExc_TypeError, "could not convert string to enum because there is no GType associated to look up the value");
+	    res = -1;
+	}
+	info = g_enum_get_value_by_name(eclass, str);
+	g_type_class_unref(eclass);
 
 	if (!info)
 	    info = g_enum_get_value_by_nick(eclass, str);
@@ -544,14 +560,13 @@ pyg_enum_get_value(GType enum_type, PyObject *obj, gint *val)
 	PyErr_SetString(PyExc_TypeError,"enum values must be strings or ints");
 	res = -1;
     }
-    g_type_class_unref(eclass);
     return res;
 }
 
 static gint
 pyg_flags_get_value(GType flag_type, PyObject *obj, gint *val)
 {
-    GFlagsClass *fclass = G_FLAGS_CLASS(g_type_class_ref(flag_type));
+    GFlagsClass *fclass = NULL;
     gint res = -1;
 
     g_return_val_if_fail(val != NULL, -1);
@@ -562,9 +577,18 @@ pyg_flags_get_value(GType flag_type, PyObject *obj, gint *val)
 	*val = PyInt_AsLong(obj);
 	res = 0;
     } else if (PyString_Check(obj)) {
+	GFlagsValue *info;
 	char *str = PyString_AsString(obj);
-	GFlagsValue *info = g_flags_get_value_by_name(fclass, str);
 
+	if (flag_type != G_TYPE_NONE)
+	    fclass = G_FLAGS_CLASS(g_type_class_ref(flag_type));
+	else {
+	    PyErr_SetString(PyExc_TypeError, "could not convert string to flag because there is no GType associated to look up the value");
+	    res = -1;
+	}
+	info = g_flags_get_value_by_name(fclass, str);
+	g_type_class_unref(fclass);
+	
 	if (!info)
 	    info = g_flags_get_value_by_nick(fclass, str);
 	if (info) {
@@ -580,6 +604,14 @@ pyg_flags_get_value(GType flag_type, PyObject *obj, gint *val)
 	len = PyTuple_Size(obj);
 	*val = 0;
 	res = 0;
+
+	if (flag_type != G_TYPE_NONE)
+	    fclass = G_FLAGS_CLASS(g_type_class_ref(flag_type));
+	else {
+	    PyErr_SetString(PyExc_TypeError, "could not convert string to flag because there is no GType associated to look up the value");
+	    res = -1;
+	}
+
 	for (i = 0; i < len; i++) {
 	    PyObject *item = PyTuple_GetItem(obj, i);
 	    char *str = PyString_AsString(item);
@@ -595,12 +627,12 @@ pyg_flags_get_value(GType flag_type, PyObject *obj, gint *val)
 		break;
 	    }
 	}
+	g_type_class_unref(fclass);
     } else {
 	PyErr_SetString(PyExc_TypeError,
 			"flag values must be strings, ints or tuples");
 	res = -1;
     }
-    g_type_class_unref(fclass);
     return res;
 }
 
@@ -943,10 +975,10 @@ pyg_closure_marshal(GClosure *closure,
     }
     ret = PyObject_CallObject(pc->callback, params);
     if (ret == NULL) {
-	/* XXXX - do fatal exceptions thing here */
-	PyErr_Print();
-	PyErr_Clear();
-	/* XXXX - clean up if threading was used */
+	if (!pyg_fatal_exceptions_notify()) {
+	    PyErr_Print();
+	    PyErr_Clear();
+	}
 	return;
     }
     if (return_value)
@@ -978,7 +1010,7 @@ pyg_closure_new(PyObject *callback, PyObject *extra_args, PyObject *swap_data)
     }
     if (swap_data) {
 	Py_INCREF(swap_data);
-	((PyGClosure *)closure)->swap_data;
+	((PyGClosure *)closure)->swap_data = swap_data;
 	closure->derivative_flag = TRUE;
     }
     return closure;
@@ -1056,9 +1088,10 @@ pyg_signal_class_closure_marshal(GClosure *closure,
 
     ret = PyObject_CallObject(method, params);
     if (ret == NULL) {
-	/* XXXX - do fatal exceptions thing here */
-	PyErr_Print();
-	PyErr_Clear();
+	if (!pyg_fatal_exceptions_notify()) {
+	    PyErr_Print();
+	    PyErr_Clear();
+	}
 	/* XXXX - clean up if threading was used */
 	Py_DECREF(method);
 	return;
@@ -1102,6 +1135,18 @@ pygobject_dealloc(PyGObject *self)
 	g_object_set_qdata_full(obj, pygobject_ownedref_key,
 				self, pygobject_destroy_notify);
 	g_object_unref(obj);
+
+	/* we ref the type, so subtype_dealloc() doesn't kill off our
+         * instance's type. */
+	if (self->ob_type->tp_flags & Py_TPFLAGS_HEAPTYPE)
+	    Py_INCREF(self->ob_type);
+	
+#ifdef Py_TRACE_REFS
+	/* if we're tracing refs, set up the reflist again, as it was just
+	 * torn down */
+        _Py_NewReference((PyObject *) self);
+#endif
+
 	return;
     }
     if (obj && !self->hasref) /* don't unref the GObject if it owns us */
@@ -1632,7 +1677,7 @@ static PyGetSetDef pygobject_getsets[] = {
 static PyTypeObject PyGObject_Type = {
     PyObject_HEAD_INIT(NULL)
     0,					/* ob_size */
-    "GObject",				/* tp_name */
+    "gobject.GObject",			/* tp_name */
     sizeof(PyGObject),			/* tp_basicsize */
     0,					/* tp_itemsize */
     /* methods */
@@ -1672,6 +1717,7 @@ static PyTypeObject PyGObject_Type = {
     PyType_GenericAlloc,		/* tp_alloc */
     PyType_GenericNew,			/* tp_new */
     object_gc_free,			/* tp_free */
+    (inquiry)0,				/* tp_is_gc */
     (PyObject *)0,			/* tp_bases */
 };
 
@@ -1693,7 +1739,7 @@ pyg_interface_init(PyObject *self, PyObject *args, PyObject *kwargs)
 static PyTypeObject PyGInterface_Type = {
     PyObject_HEAD_INIT(NULL)
     0,                                  /* ob_size */
-    "GInterface",                       /* tp_name */
+    "gobject.GInterface",               /* tp_name */
     sizeof(PyObject),                   /* tp_basicsize */
     0,                                  /* tp_itemsize */
     /* methods */
@@ -1732,6 +1778,7 @@ static PyTypeObject PyGInterface_Type = {
     PyType_GenericAlloc,		/* tp_alloc */
     PyType_GenericNew,			/* tp_new */
     object_free,			/* tp_free */
+    (inquiry)0,				/* tp_is_gc */
     (PyObject *)0,			/* tp_bases */
 };
 
@@ -1752,7 +1799,6 @@ pyg_register_interface(PyObject *dict, const gchar *class_name,
     if (gtype) {
 	o = pyg_type_wrapper_new(gtype);
 	PyDict_SetItemString(type->tp_dict, "__gtype__", o);
-	PyDict_SetItemString(type->tp_defined, "__gtype__", o);
 	Py_DECREF(o);
     }
 
@@ -1950,7 +1996,6 @@ pyg_type_register(PyObject *self, PyObject *args)
     /* set new value of __gtype__ on class */
     gtype = pyg_type_wrapper_new(instance_type);
     PyDict_SetItemString(class->tp_dict, "__gtype__", gtype);
-    PyDict_SetItemString(class->tp_defined, "__gtype__", gtype);
     Py_DECREF(gtype);
 
     Py_INCREF(Py_None);
@@ -2191,13 +2236,30 @@ static PyMethodDef pygobject_functions[] = {
 
 /* ----------------- Constant extraction ------------------------ */
 
+static char *
+pyg_constant_strip_prefix(gchar *name, const gchar *strip_prefix)
+{
+    gint prefix_len;
+    guint j;
+    
+    prefix_len = strlen(strip_prefix);
+    
+    /* strip off prefix from value name, while keeping it a valid
+     * identifier */
+    for (j = prefix_len; j >= 0; j--) {
+	if (g_ascii_isalpha(name[j]) || name[j] == '_') {
+	    return &name[j];
+	}
+    }
+    return name;
+}
+
 static void
 pyg_enum_add_constants(PyObject *module, GType enum_type,
 		       const gchar *strip_prefix)
 {
     GEnumClass *eclass;
-    guint i, j;
-    gint prefix_len;
+    guint i;
 
     /* a more useful warning */
     if (!G_TYPE_IS_ENUM(enum_type)) {
@@ -2206,22 +2268,15 @@ pyg_enum_add_constants(PyObject *module, GType enum_type,
     }
     g_return_if_fail (strip_prefix != NULL);
 
-    prefix_len = strlen(strip_prefix);
     eclass = G_ENUM_CLASS(g_type_class_ref(enum_type));
 
     for (i = 0; i < eclass->n_values; i++) {
 	gchar *name = eclass->values[i].value_name;
 	gint value = eclass->values[i].value;
 
-	/* strip off prefix from value name, while keeping it a valid
-         * identifier */
-	for (j = prefix_len; j >= 0; j--) {
-	    if (g_ascii_isalpha(name[j]) || name[j] == '_') {
-		name = &name[j];
-		break;
-	    }
-	}
-	PyModule_AddIntConstant(module, name, (long) value);
+	PyModule_AddIntConstant(module,
+				pyg_constant_strip_prefix(name, strip_prefix),
+				(long) value);
     }
 
     g_type_class_unref(eclass);
@@ -2232,8 +2287,7 @@ pyg_flags_add_constants(PyObject *module, GType flags_type,
 			const gchar *strip_prefix)
 {
     GFlagsClass *fclass;
-    guint i, j;
-    gint prefix_len;
+    guint i;
 
     /* a more useful warning */
     if (!G_TYPE_IS_FLAGS(flags_type)) {
@@ -2242,27 +2296,46 @@ pyg_flags_add_constants(PyObject *module, GType flags_type,
     }
     g_return_if_fail (strip_prefix != NULL);
 
-    prefix_len = strlen(strip_prefix);
     fclass = G_FLAGS_CLASS(g_type_class_ref(flags_type));
 
     for (i = 0; i < fclass->n_values; i++) {
 	gchar *name = fclass->values[i].value_name;
 	guint value = fclass->values[i].value;
 
-	/* strip off prefix from value name, while keeping it a valid
-         * identifier */
-	for (j = prefix_len; j >= 0; j--) {
-	    if (g_ascii_isalpha(name[j]) || name[j] == '_') {
-		name = &name[j];
-		break;
-	    }
-	}
-	PyModule_AddIntConstant(module, name, (long) value);
+	PyModule_AddIntConstant(module,
+				pyg_constant_strip_prefix(name, strip_prefix),
+				(long) value);
     }
 
     g_type_class_unref(fclass);
 }
 
+static int
+pyg_fatal_exceptions_notify(void)
+{
+    GList *tmp_list = pygobject_exception_notifiers;
+    if (!tmp_list)
+	return 0;
+    while (tmp_list != NULL) {
+	PyGFatalExceptionFunc notifier = tmp_list->data;
+	(* notifier)();
+	tmp_list = g_list_next (tmp_list);
+    }
+    return 1;
+}
+static void
+pyg_fatal_exceptions_notify_add(PyGFatalExceptionFunc func)
+{
+    pygobject_exception_notifiers = 
+	g_list_append(pygobject_exception_notifiers, func);
+}
+
+static void
+pyg_fatal_exceptions_notify_remove(PyGFatalExceptionFunc func)
+{
+    pygobject_exception_notifiers = 
+	g_list_remove(pygobject_exception_notifiers, func);
+}
 
 /* ----------------- gobject module initialisation -------------- */
 
@@ -2288,6 +2361,11 @@ static struct _PyGObject_Functions functions = {
 
   pyg_enum_add_constants,
   pyg_flags_add_constants,
+  
+  pyg_fatal_exceptions_notify_add,
+  pyg_fatal_exceptions_notify_remove,
+
+  pyg_constant_strip_prefix,
 };
 
 DL_EXPORT(void)
@@ -2316,7 +2394,6 @@ initgobject(void)
     PyDict_SetItemString(d, "GInterface", (PyObject *)&PyGInterface_Type);
     PyDict_SetItemString(PyGInterface_Type.tp_dict, "__gtype__",
 			 o=pyg_type_wrapper_new(G_TYPE_INTERFACE));
-    PyDict_SetItemString(PyGInterface_Type.tp_defined, "__gtype__", o);
     Py_DECREF(o);
 
     PyGBoxed_Type.ob_type = &PyType_Type;
@@ -2325,7 +2402,6 @@ initgobject(void)
     PyDict_SetItemString(d, "GBoxed", (PyObject *)&PyGBoxed_Type);
     PyDict_SetItemString(PyGBoxed_Type.tp_dict, "__gtype__",
 			 o=pyg_type_wrapper_new(G_TYPE_BOXED));
-    PyDict_SetItemString(PyGBoxed_Type.tp_defined, "__gtype__", o);
     Py_DECREF(o);
 
     boxed_marshalers = g_hash_table_new(g_direct_hash, g_direct_equal);
